/* EPS Conduits (C) 2014-2017 Stephane Charette <stephanecharette@gmail.com>
 * $Id: TCPConnection.hpp 2167 2017-03-01 12:10:04Z stephane $
 * See license.txt.
 */

#pragma once

#include "Connection.hpp"
#include <boost/asio/ip/tcp.hpp>
#include <boost/uuid/nil_generator.hpp>
#include <set>
#include <memory>


namespace EPS
{
	/// TCP connection.
	class TCPConnection final : public EPS::Connection
	{
		public:

			/// Constructor. @{
			TCPConnection(void);
			TCPConnection(const EPS::TCPEP &eth_endpoint, const EPS::UUID &uuid = boost::uuids::nil_uuid());
			/// @}

			/// Destructor.
			virtual ~TCPConnection(void);

			/// Perform an asynchronous connect to the endpoint specified when the object was constructed.
			virtual void async_connect(void);

			/// Call when a connection is accepted to refresh some of the internal values.
			virtual void accepted(void);

			/// Set various socket options.
			virtual void set_socket_options(void);

			bool								is_mesh;	///< is this connection a client<->server connection, or part of the mesh network?
			boost::asio::ip::tcp::socket		socket;		///< TCP socket for this TCP connection.
			TCPEP								endpoint;	///< TCP endpoint.

			/// Calls to async_write() cannot overlap, so we must store NetBuffers until they can be sent.
			EPS::NBDeque						pending_write_deque;
	};

	/** TCP connections are dynamically allocated when a socket is needed, so this type allows us to easily manage the
	 * lifespan of all connections.  @see EPS::UConnPtr
	 */
	typedef std::shared_ptr<EPS::TCPConnection> TConnPtr;

	/// When multiple TCP connections need to be grouped.
	typedef std::set<TConnPtr> TCPConns;
}
