#!/usr/bin/env bash
#
# EPS Conduits (C) 2014 Stephane Charette <stephanecharette@gmail.com>
# $Id: eps-client.sh 1831 2016-05-06 16:25:21Z stephane $


if [ "${EUID}" != "0" ]; then
	logger -i -p daemon.err -s "EPS Conduits client typically needs to be run as root. (Now running as \"${USER}\".)"
fi

if [ "$(which kldload)" != "" ]; then
	# on FreeBSD, we need to manually load if_tap
	kldload if_tap &>/dev/null
fi

# prevent tight loops if something ever goes wrong, and give the system
# time to reboot since this script is also called by crond
#
# (sleep for slightly more than the server in case both the client and
# server are running on the same device)
sleep 20

logger -i -p daemon.info "Restarting EPS Conduits client."

# This software is likely to be installed on devices that are very
# remote, or where access is very limited, so if EPS hangs in any
# way, it can be costly/difficult to fix.  For this reason, we'll
# start it with a timeout to ensure that it restarts itself every
# 24 hours.  Should something go wrong, then within 24 hours the
# device should be back on the EPS network.

# use timeout only if it is installed, otherwise we cross our fingers
# and hope that everything works ok without the regular restarts :)
TIMEOUT=$(which timeout)
if [ "${TIMEOUT}" != "" ]; then
	TIMEOUT="${TIMEOUT} --signal=TERM --kill-after=60s 24h"
fi

${TIMEOUT} eps-client --conf /etc/eps.conf

# reload/restart the script (useful to pick up changes after an upgrade)
exec $0 $@

