#!/usr/bin/env bash
#
# EPS Conduits (C) 2014 Stephane Charette <stephanecharette@gmail.com>
# $Id: eps-configure.sh 1859 2016-05-24 01:13:37Z stephane $


LF=1
SC=1
GV=1
stats=1
AUTOSTART_SERVER=1
AUTOSTART_CLIENT=1


# Ubuntu and FreeBSD uses 'dialog'
DLG=$(which dialog)
if [ "${DLG}" == "" ]; then
	# Redhat/CentOS/Fedora uses 'whiptail'
	DLG=$(which whiptail)
	if [ "${DLG}" == "" ]; then
		echo "This script requires either 'dialog' or 'whiptail'."
		exit 1
	fi
fi


####################
# CONFIGURATION FILE
####################

if [ "${EUID}" != "0" ]; then
	${DLG} --backtitle "EPS Conduits" --title "Warning" --msgbox "This script needs to be run as 'sudo'." 0 0
	echo "Aborted EPS Conduits configuration."
	exit 1
fi

EPS_CONF=/etc/eps.conf
if [ "$1" != "" ]; then
	EPS_CONF=$1
fi

EPS_CONF=$( ${DLG} --backtitle "EPS Conduits" --title "Configuration File" --nocancel --inputbox "Which EPS Conduits configuration file do you want to edit?" 0 0 "${EPS_CONF}" 3>&1 1>&2 2>&3)
if [ "${EPS_CONF}" == "" ]; then
	EPS_CONF=/etc/eps.conf
fi
if [ ! -e "${EPS_CONF}" ]; then
	${DLG} --backtitle "EPS Conduits" --title "${EPS_CONF}" --yesno "The EPS Conduits configuration file ${EPS_CONF} does not exist.\nDo you want to create a new file?" 0 0
	if [ $? -ne 0 ]; then
		echo "Aborted EPS Conduits configuration."
		exit 1
	fi
else
	TXT=$(cat <<-'END_HEREDOC'
		Some advanced or obscure EPS Conduits feature configuration
		options are not covered by this configuration tool, and may be
		dropped from the configuration file if they have been manually
		configured.

		Do you want to continue with this configuration tool?
	END_HEREDOC
	)
	${DLG} --backtitle "EPS Conduits" --title "${EPS_CONF}" --yesno "${TXT}" 0 0
	if [ $? -ne 0 ]; then
		echo "Aborted EPS Conduits configuration."
		exit 1
	fi

	#
	# SOURCE THE EXISTING CONFIG FILE
	#
	. ${EPS_CONF}
fi


logger -i -p daemon.info "re-configuring EPS Conduits (${USER}:${SUDO_USER:-?}:${EPS_CONF})"


#
# CLIENT, SERVER, OR BOTH?
#
CLIENT_ONLY=on
SERVER_ONLY=off
CLIENT_AND_SERVER=off
CLIENT_PID=$(pidof eps-client)
SERVER_PID=$(pidof eps-server)
if [ "${CLIENT_PID}" != "" -a "${SERVER_PID}" != "" ]; then
	CLIENT_ONLY=off
	SERVER_ONLY=off
	CLIENT_AND_SERVER=on
elif [ "${SERVER_PID}" != "" ]; then
	CLIENT_ONLY=off
	SERVER_ONLY=on
	CLIENT_AND_SERVER=off
fi
MODE=$( ${DLG} --backtitle "EPS Conduits" --title "EPS Mode" --nocancel --radiolist "How will you run EPS Conduits on this computer?" 0 0 0 1 "EPS Client only" ${CLIENT_ONLY} 2 "EPS Server only" ${SERVER_ONLY} 3 "Both EPS Client and EPS Server" ${CLIENT_AND_SERVER} 3>&1 1>&2 2>&3)


################
# COMMON OPTIONS
################

#
# PORT
#
TXT=$(cat <<-'END_HEREDOC'
	The EPS clients interconnect to form a mesh network.  The TCP port
	configuration setting determines where EPS clients are expected to
	connect to each other when creating the mesh network.

	In addition, this also describes the TCP port to which the server
	binds, listening for new EPS clients to connect.

	If you modify the default, then it must be modified both in the
	server's configuration file as well as all of the clients that are
	expected to connect to that network.

	Normally, the tcp port configuration setting defaults to 43439.
END_HEREDOC
)
port=$( ${DLG} --backtitle "EPS Conduits -- common options" --title "TCP Port" --nocancel --inputbox "${TXT}" 0 0 "${port:-43439}" 3>&1 1>&2 2>&3)


#
# LOG
#
TXT=$(cat <<-'END_HEREDOC'
	The log files are normally written to /var/log/eps*.log.  By
	default, the EPS log files are automatically rotated as
	necessary.

	Do you want to customize where the log files are written?

	Normally, you should select 'no' to accept the default location.
END_HEREDOC
)
${DLG} --backtitle "EPS Conduits -- common options" --title "Log File" --defaultno --yesno "${TXT}" 0 0
LF=$?
if [ ${LF} -eq 0 ]; then
	logfile=$( ${DLG} --backtitle "EPS Conduits -- common options" --title "Log File" --nocancel --inputbox "Where should the log file be written?" 0 0 "${logfile:-/var/log/eps.log}" 3>&1 1>&2 2>&3)
	if [ "${logfile}" == "" ]; then
		logfile=/var/log/eps.log
	fi
fi


#
# SCORECARD
#
TXT=$(cat <<-'END_HEREDOC'
	Transfer and bandwidth statistics for EPS Conduits can be written
	out to a 'scorecard' file every 60 seconds.  For example, the
	scorecard file can then be easily read into other tools or sourced
	and processed by simple scripts to generate graphs or health checks.

	Do you want to enable scorecard support?  The default value is 'yes'.
END_HEREDOC
)
${DLG} --backtitle "EPS Conduits -- common options" --title "Statistics Scorecard" --yesno "${TXT}" 0 0
SC=$?
if [ ${SC} -eq 0 ]; then
	scorecard=$( ${DLG} --backtitle "EPS Conduits -- common options" --title "Statistics Scorecard" --nocancel --inputbox "Where should the statistics scorecard file be written?" 0 0 "${scorecard:-/var/log/eps.stats}" 3>&1 1>&2 2>&3)
	if [ "${scorecard}" == "" ]; then
		scorecard=/var/log/eps.stats
	fi
fi


#
# STATS
#
TXT=$(cat <<-'END_HEREDOC'
	The 'stats' configuration option is used to tell EPS Conduits to
	dump a record of the statistics to the log file every 60 seconds.
	This can cause the log file to grow relatively quickly.
	
	Do you want to enable stats output?  The default value is 'no'.
END_HEREDOC
)
${DLG} --backtitle "EPS Conduits -- common options" --title "Stats Output To Log" --defaultno --yesno "${TXT}" 0 0
stats=$?


################
# SERVER OPTIONS
################
if [ ${MODE} -eq 2 -o ${MODE} -eq 3 ]; then

#
# GRAPHVIZ
#
TXT=$(cat <<-'END_HEREDOC'
	The 'graphviz' configuration option tells the EPS server to write a
	network diagram with network statistics every 5 minutes.  The usual
	Graphviz tools (such as circo) can then be used to render the diagram
	as a PNG image.
	
	Do you want to enable Graphviz network diagrams?  The default value
	is 'yes'.
END_HEREDOC
)
${DLG} --backtitle "EPS Conduits -- server options" --title "Graphviz Network Diagram" --yesno "${TXT}" 0 0
GV=$?
if [ ${GV} -eq 0 ]; then
	graphviz=$( ${DLG} --backtitle "EPS Conduits -- server options" --title "Graphviz Network Diagram" --nocancel --inputbox "Where should the Graphviz network diagram be saved?" 0 0 "${graphviz:-/var/log/eps_network.gz}" 3>&1 1>&2 2>&3)
fi


#
# NETWORK
#
TXT=$(cat <<-'END_HEREDOC'
	The network configuration option indicates the range from which
	addresses will be assigned to EPS clients. The network must be 24-bit.
	
	For example, valid 'network' values would be 192.168.99.0/24 or
	10.0.1.0/24.  The default value is 192.168.43.0/24.
END_HEREDOC
)
network=$( ${DLG} --backtitle "EPS Conduits -- server options" --title "EPS Network" --nocancel --inputbox "${TXT}" 0 0 "${network:-192.168.43.0/24}" 3>&1 1>&2 2>&3)


#
# UPNPTIMEOUT
#
TXT=$(cat <<-'END_HEREDOC'
	The 'upnptimeout' configuration option determines how long (in
	milliseconds) the EPS server will wait on startup to look for a
	UPnP-capable router or NAT device.  Set to zero to disable UPnP
	support.
	
	How long should the EPS server wait for UPnP?  The default value
	is 1000 milliseconds.
END_HEREDOC
)
upnptimeout=$( ${DLG} --backtitle "EPS Conduits -- server options" --title "UPnP Timeout" --nocancel --inputbox "${TXT}" 0 0 "${upnptimeout:-1000}" 3>&1 1>&2 2>&3)


#
# AUTOSTART (/etc/cron.d/)
#
if [ -d /etc/cron.d -o -e /etc/crontab ]; then
	TXT=$(cat <<-'END_HEREDOC'
		Should a cron job be created to start eps-server.sh every time
		the computer is rebooted?
	END_HEREDOC
	)
	${DLG} --backtitle "EPS Conduits -- server options" --title "Automatically Start EPS Server" --yesno "${TXT}" 0 0
	AUTOSTART_SERVER=$?
fi


fi
###############
# END OF SERVER
###############


################
# CLIENT OPTIONS
################
if [ ${MODE} -eq 1 -o ${MODE} -eq 3 ]; then


#
# SERVER
#
TXT=$(cat <<-'END_HEREDOC'
	The server configuration option indicates the location of the
	EPS server to which the EPS client should connect.  The server
	can be either an IP address or a hostname.
	
	For example, 1.2.3.4 or a hostname such as eps.example.com.
END_HEREDOC
)
server=$( ${DLG} --backtitle "EPS Conduits -- client options" --title "EPS Server Address" --nocancel --inputbox "${TXT}" 0 0 "${server:-eps.example.com}" 3>&1 1>&2 2>&3)


#
# AUTOSTART (/etc/cron.d/)
#
if [ -d /etc/cron.d -o -e /etc/crontab ]; then
	TXT=$(cat <<-'END_HEREDOC'
		Should a cron job be created to start eps-client.sh every
		time the computer is rebooted?
	END_HEREDOC
	)
	${DLG} --backtitle "EPS Conduits -- client options" --title "Automatically Start EPS Client" --yesno "${TXT}" 0 0
	AUTOSTART_CLIENT=$?
fi


fi
###############
# END OF CLIENT
###############


################
# CIPHER OPTIONS
################
TXT=$(cat <<-'END_HEREDOC'
	All EPS clients and the EPS server must share the exact same
	128-bit encryption cipher IV to communicate.  Normally, the
	cipher values are first set on the EPS server (the IV is
	randomly generated if not set) and then must be manually
	copied to each EPS client.
END_HEREDOC
)
cipher_iv=$( ${DLG} --backtitle "EPS Conduits -- cipher options" --title "EPS encryption cipher IV"  --nocancel --inputbox "${TXT}"  0 0 "${cipher_iv}"  3>&1 1>&2 2>&3)

TXT=$(cat <<-'END_HEREDOC'
	All EPS clients and the EPS server must share the exact same 256-bit
	encryption cipher KEY to communicate.  Normally, the cipher values are
	first set on the EPS server (the KEY is randomly generated if not set)
	and then must be manually copied to each EPS client.
END_HEREDOC
)
cipher_key=$( ${DLG} --backtitle "EPS Conduits -- cipher options" --title "EPS encryption cipher KEY" --nocancel --inputbox "${TXT}" 0 0 "${cipher_key}" 3>&1 1>&2 2>&3)


${DLG} --backtitle "EPS Conduits" --title "${EPS_CONF}" --yesno "Save the EPS configuration settings?" 0 0
SAVE=$?
if [ ${SAVE} -ne 0 ]; then
	echo "Aborted EPS Conduits configuration."
	exit 1
fi

OUTPUT=${EPS_CONF}_$$

logger -i -p daemon.info "writing new EPS Conduits configuration file (${USER}:${SUDO_USER:-?}:${EPS_CONF})"

echo "# EPS Conduits configuration file written by $0" > ${OUTPUT}
echo "# Saved: $(date --rfc-2822)"			>> ${OUTPUT}
echo "# User: ${USER} / ${SUDO_USER:-?}"	>> ${OUTPUT}
echo "#"									>> ${OUTPUT}
echo ""										>> ${OUTPUT}
echo "port=${port}"							>> ${OUTPUT}

if [ ${LF} -ne 0 ]; then
	echo -n "#"								>> ${OUTPUT}
fi
echo "logfile=${logfile}"					>> ${OUTPUT}

if [ ${SC} -ne 0 ]; then
	echo -n "#"								>> ${OUTPUT}
fi
echo "scorecard=${scorecard}"				>> ${OUTPUT}

if [ ${GV} -ne 0 ]; then
	echo -n "#"								>> ${OUTPUT}
fi
echo "graphviz=${graphviz}"					>> ${OUTPUT}

if [ ${stats} -ne 0 ]; then
	echo -n "#"								>> ${OUTPUT}
fi
echo "stats="								>> ${OUTPUT}

echo "upnptimeout=${upnptimeout}"			>> ${OUTPUT}
echo "network=${network}"					>> ${OUTPUT}
echo "#sequential="							>> ${OUTPUT}
echo "server=${server}"						>> ${OUTPUT}
echo "cipher_iv=${cipher_iv}"				>> ${OUTPUT}
echo "cipher_key=${cipher_key}"				>> ${OUTPUT}
echo "client_uuid=${client_uuid}"			>> ${OUTPUT}
echo "server_uuid=${server_uuid}"			>> ${OUTPUT}

mv -f "${OUTPUT}" "${EPS_CONF}"
chmod 600 "${EPS_CONF}"


##########
# CRON JOB
##########

CRONJOB=""
if [ -d /etc/cron.d ]; then
	CRONJOB=/etc/cron.d/eps-conduits
	echo "# EPS Conduits cron job file written by $0"							>  "${CRONJOB}"
	echo "# Saved: $(date --rfc-2822)"											>> "${CRONJOB}"
	echo "# User: ${USER} / ${SUDO_USER:-?}"									>> "${CRONJOB}"
	echo ""																		>> "${CRONJOB}"
	echo "MAILTO=\"\""															>> "${CRONJOB}"
	echo "SHELL=/bin/bash"														>> "${CRONJOB}"
	echo "PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin"	>> "${CRONJOB}"
	echo ""																		>> "${CRONJOB}"
elif [ -e /etc/crontab ]; then
	# for example, FreeBSD just has crontab, not cron.d
	CRONJOB=/etc/crontab
	CRONJOB_TMP="${CRONJOB}_$$"
	grep -v -E "eps-client|eps-server" "${CRONJOB}" > "${CRONJOB_TMP}"
	mv -f "${CRONJOB_TMP}" "${CRONJOB}"
fi

if [ "${CRONJOB}" != "" ]; then
	if [ ${AUTOSTART_SERVER} -ne 0 ]; then
		echo -n "# " >> "${CRONJOB}"
	fi
	echo "@reboot	root	$(which eps-server.sh)" >> "${CRONJOB}"
	if [ ${AUTOSTART_CLIENT} -ne 0 ]; then
		echo -n "# " >> "${CRONJOB}"
	fi
	echo "@reboot	root	$(which eps-client.sh)" >> "${CRONJOB}"
	echo "EPS Conduits cron job written to ${CRONJOB}."
fi


#########
# RESTART
#########
# If either of the EPS binaries are running, then offer to send a HUP signal to restart them
EPS_PIDS=$(pidof eps-client eps-server)
if [ "${EPS_PIDS}" != "" ]; then
	${DLG} --backtitle "EPS Conduits" --title "Restart EPS Conduits?" --yesno "Restart EPS Conduits?" 0 0
	if [ $? -eq 0 ]; then
		logger -i -p daemon.info "user-requested restart of EPS Conduits (${USER}:${SUDO_USER:-?})"
		# "quiet" means do not complain if a process doesn't exist, "verbose" makes it print a line for every successful kill
		echo ""
		killall --signal HUP --quiet --verbose eps-client eps-server
		echo "EPS Conduits restarted."
	else
		echo ""
		echo "EPS Conduits has not been restarted."
	fi
else
	TXT=$(cat <<-'END_HEREDOC'
		EPS Conduits doesn't seem to be running.  If a cron job has just been
		added to automatically start EPS Conduits on bootup, this should take
		effect the next time this system is rebooted.  Otherwise, to manually
		start EPS Conduits, see eps-client.sh and eps-server.sh.
		
		For example, to start the client manually:
		
		        sudo eps-client.sh
		
		For more examples, also see the cron job in /etc/crontab or in
		/etc/cron.d/eps-conduits.
	END_HEREDOC
	)
	${DLG} --backtitle "EPS Conduits" --title "EPS Conduits Not Started" --msgbox "${TXT}" 0 0
	echo ""
	echo "EPS Conduits are not running."
fi

echo "EPS Conduits configuration file saved to ${EPS_CONF}."
cat "${EPS_CONF}"

echo ""

exit 0
